/* -*-c-*-
 *
 * $Id: ectab.h,v 1.3 2004/04/01 21:28:41 mdw Exp $
 *
 * Table of standard elliptic curves
 *
 * (c) 2004 Straylight/Edgeware
 */

/*----- Licensing notice --------------------------------------------------* 
 *
 * This file is part of Catacomb.
 *
 * Catacomb is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 * 
 * Catacomb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU Library General Public
 * License along with Catacomb; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

/*----- Revision history --------------------------------------------------* 
 *
 * $Log: ectab.h,v $
 * Revision 1.3  2004/04/01 21:28:41  mdw
 * Normal basis support (translates to poly basis internally).  Rewrite
 * EC and prime group table generators in awk, so that they can reuse data
 * for repeated constants.
 *
 * Revision 1.2  2004/04/01 12:50:09  mdw
 * Add cyclic group abstraction, with test code.  Separate off exponentation
 * functions for better static linking.  Fix a buttload of bugs on the way.
 * Generally ensure that negative exponents do inversion correctly.  Add
 * table of standard prime-field subgroups.  (Binary field subgroups are
 * currently unimplemented but easy to add if anyone ever finds a good one.)
 *
 * Revision 1.1  2004/03/27 17:54:11  mdw
 * Standard curves and curve checking.
 *
 */

#ifndef CATACOMB_ECTAB_H
#define CATACOMB_ECTAB_H

#ifdef __cplusplus
  extern "C" {
#endif

/*----- Header files ------------------------------------------------------*/
    
#include "ec.h"

/*----- Data structures ---------------------------------------------------*/

typedef struct ecdata {
  unsigned ftag;			/* The kind of curve this is */
  mp p, beta;				/* Modulus, and conversion magic */
  mp a, b;				/* Elliptic curve parameters */
  mp r;					/* Order of common point %$g$% */
  mp h;					/* Cofactor %$h = \#E/r$% */
  mp gx, gy;				/* Common point */
} ecdata;

enum {
  FTAG_PRIME,				/* Prime but not nice */
  FTAG_NICEPRIME,			/* Nice prime field */
  FTAG_BINPOLY,				/* Binary field, poly basis */
  FTAG_BINNORM				/* Binary field, normal basis */
};

typedef struct ecentry {
  const char *name;
  ecdata *data;
} ecentry;

/*----- Global variables --------------------------------------------------*/

extern const ecentry ectab[];

/*----- That's all, folks -------------------------------------------------*/

#ifdef __cplusplus
  }
#endif

#endif
